@@ -552,7 +552,7 @@ ibus_m17n_engine_update_preedit (IBusM17NEngine *m17n)
 
     buf = ibus_m17n_mtext_to_utf8 (m17n->context->preedit);
     if (buf) {
-        text = ibus_text_new_from_static_string (buf);
+        text = ibus_text_new_from_string (buf);
         if (klass->preedit_foreground != INVALID_COLOR)
             ibus_text_append_attribute (text, IBUS_ATTR_TYPE_FOREGROUND,
                                         klass->preedit_foreground, 0, -1);
@@ -567,6 +567,7 @@ ibus_m17n_engine_update_preedit (IBusM17NEngine *m17n)
                                                    mtext_len (m17n->context->preedit) > 0,
                                                    klass->preedit_focus_mode);
     }
+    g_free (buf);
 }
 
 static void
@@ -574,8 +575,22 @@ ibus_m17n_engine_commit_string (IBusM17NEngine *m17n,
                                 const gchar    *string)
 {
     IBusText *text;
-    text = ibus_text_new_from_static_string (string);
+    text = ibus_text_new_from_string (string);
     ibus_engine_commit_text ((IBusEngine *)m17n, text);
+    /*
+      Updating the preedit after commit is necessary because some
+      applications (OpenOffice.org or Evolution) expect that
+      "preedit-changed" is signalled after "commit" and clear
+      applications' preedit buffers on "commit".  However, ibus-m17n
+      (possibly other IME) may signal "commit" just after
+      "preedit-changed". So we need to make sure to signal
+      "preedit-changed" after "commit".
+
+      Test case: type "iupap,h" on oowriter with
+      "m17n:si:wijesekera - si-wijesekera (m17n)".
+      You won't see the n+1’th character in the preedit buffer updating
+      the preedit after commit.
+    */
     ibus_m17n_engine_update_preedit (m17n);
 }
 
@@ -591,7 +606,7 @@ ibus_m17n_engine_commit_string (IBusM17NEngine *m17n,
        if (code == altgr)
          ...
      }
-                
+
    Since IBus engines are supposed to be cross-platform, the code
    should go into IBus core, instead of ibus-m17n. */
 static MSymbol
@@ -732,20 +747,44 @@ ibus_m17n_engine_process_key_event (IBusEngine     *engine,
         break;
     }
 
+    /*
+      Compose support:
+
+      IBusM17NEngine inherits from
+      IBusEngineSimple. IBUS_ENGINE_CLASS(parent_class)->process_key_event()
+      calls ibus_engine_simple_process_key_event(). If this does not
+      start to process a compose sequence, it calls
+      ibus_engine_hide_preedit_text((IBusEngine *)simple) before
+      returning FALSE. If m17n->context->preedit was not empty when
+      this happens, the preedit of IBusM17NEngine disappears for a
+      very short moment until it is shown again later when
+      IBusM17NEngine continues processing the key event. This may
+      cause a flicker in the preedit.  How visible this flicker is
+      depends on the circumstances, but sometimes it can be quite
+      visible and annoying. It seems possible to avoid 99% of the
+      flicker by calling ibus_engine_show_preedit_text ((IBusEngine
+      *)m17n) directly before **and** after
+      IBUS_ENGINE_CLASS(parent_class)->process_key_event() if the
+      preedit of IBusM17NEngine is not empty.
+    */
+    if (mtext_len (m17n->context->preedit) > 0)
+        ibus_engine_show_preedit_text ((IBusEngine *)m17n);
     if (IBUS_ENGINE_CLASS (parent_class)->process_key_event (engine, keyval, keycode, modifiers)) {
-      if (mtext_len (m17n->context->preedit) > 0) {
-	gchar *buf;
-	buf = ibus_m17n_mtext_to_utf8 (m17n->context->preedit);
-	if (buf) {
-	  IBusText *text;
-	  text = ibus_text_new_from_string (buf);
-	  ibus_engine_commit_text (engine, text);
-	  g_free (buf);
-	}
-	minput_reset_ic (m17n->context);
-      }
-      return TRUE;
+        if (mtext_len (m17n->context->preedit) > 0) {
+            gchar *buf;
+            buf = ibus_m17n_mtext_to_utf8 (m17n->context->preedit);
+            if (buf) {
+                IBusText *text;
+                text = ibus_text_new_from_string (buf);
+                ibus_engine_commit_text (engine, text);
+                g_free (buf);
+            }
+            minput_reset_ic (m17n->context);
+        }
+        return TRUE;
     }
+    if (mtext_len (m17n->context->preedit) > 0)
+        ibus_engine_show_preedit_text ((IBusEngine *)m17n);
 
     if (modifiers & IBUS_RELEASE_MASK)
         return FALSE;
